@@ -4,11 +4,10 @@ In this module lives the type tree.
 from collections import defaultdict
 from typing import Any, DefaultDict, Dict, List, Optional, Tuple, Type, Union
 
-
+import dask.dataframe as dd
 import numpy as np
 import pandas as pd
-import dask.dataframe as dd
-
+from ..clean import validate_country, validate_lat_long
 from ..errors import UnreachableError
 
 CATEGORICAL_NUMPY_DTYPES = [np.bool, np.object]
@@ -106,6 +105,28 @@ class Text(Nominal):
     """
 
 
+class GeoGraphy(Categorical):
+    """
+    Type GeoGraphy, Subtype of Categorical
+    """
+
+
+class GeoPoint(DType):
+    """
+    Type GeoPoint
+    """
+
+
+class LatLong(GeoPoint):
+    """
+    Type LatLong, Tuple
+    """
+
+    def __init__(self, lat_col: str, long_col: str) -> None:
+        self.lat = lat_col
+        self.long = long_col
+
+
 ############## End of the Type Tree ##############
 
 DTypeOrStr = Union[DType, Type[DType], str, None]
@@ -116,6 +137,7 @@ DTypeDef = Union[Dict[str, Union[DType, Type[DType], str]], DType, Type[DType],
 def detect_dtype(
     col: dd.Series,
     known_dtype: Optional[DTypeDef] = None,
+    detect_small_distinct: bool = True,
 ) -> DType:
     """
     Given a column, detect its type or transform its type according to users' specification
@@ -129,9 +151,11 @@ def detect_dtype(
         all columns. E.g.  known_dtype = {"a": Continuous, "b": "Nominal"} or
         known_dtype = {"a": Continuous(), "b": "nominal"} or
         known_dtype = Continuous() or known_dtype = "Continuous" or known_dtype = Continuous()
+    detect_small_distinct: bool, default True
+        Whether to detect numerical columns with small distinct values as categorical column.
     """
     if not known_dtype:
-        return detect_without_known(col)
+        return detect_without_known(col, detect_small_distinct=detect_small_distinct)
 
     if isinstance(known_dtype, dict):
         if col.name in known_dtype:
@@ -141,7 +165,7 @@ def detect_dtype(
     elif isinstance(normalize_dtype(known_dtype), DType):
         return map_dtype(normalize_dtype(known_dtype))
 
-    return detect_without_known(col)
+    return detect_without_known(col, detect_small_distinct=detect_small_distinct)
 
 
 def map_dtype(dtype: DType) -> DType:
@@ -165,15 +189,29 @@ def map_dtype(dtype: DType) -> DType:
         return dtype
 
 
-def detect_without_known(col: dd.Series) -> DType:
+def detect_without_known(col: dd.Series, detect_small_distinct: bool) -> DType:
+    # pylint: disable=too-many-return-statements
     """
     This function detects dtypes of column when users didn't specify.
     """
     if is_nominal(col.dtype):
-        return Nominal()
+        if is_geography(col):
+            return GeoGraphy()
+        if is_geopoint(col):
+            return GeoPoint()
+        else:
+            return Nominal()
 
     elif is_continuous(col.dtype):
-        return Continuous()
+        if detect_small_distinct:
+            # detect as categorical if distinct value is small
+            nuniques = col.nunique_approx().compute()
+            if nuniques < 10:
+                return Nominal()
+            else:
+                return Continuous()
+        else:
+            return Continuous()
 
     elif is_datetime(col.dtype):
         return DateTime()
@@ -181,7 +219,7 @@ def detect_without_known(col: dd.Series) -> DType:
         raise UnreachableError
 
 
-def is_dtype(dtype1: DType, dtype2: DType) -> bool:
+def is_dtype(dtype1: Any, dtype2: DType) -> bool:
     """
     This function detects if dtype2 is dtype1.
     """
@@ -236,6 +274,24 @@ def is_nominal(dtype: Any) -> bool:
         return any(isinstance(dtype, c) for c in CATEGORICAL_PANDAS_DTYPES)
 
 
+def is_geography(col: dd.Series) -> bool:
+    """
+    Given a column, return if its type is a geography type
+    """
+    geo = col.compute()[:100]
+    geo_ratio: float = np.sum(validate_country(geo)) / geo.shape[0]
+    return geo_ratio > 0.8
+
+
+def is_geopoint(col: dd.Series) -> bool:
+    """
+    Given a column, return if its type is a geopoint type
+    """
+    lat_long = pd.Series(col.compute()[:100], dtype="string")
+    lat_long_ratio: float = np.sum(validate_lat_long(lat_long)) / lat_long.shape[0]
+    return lat_long_ratio > 0.8
+
+
 def is_continuous(dtype: Any) -> bool:
     """
     Given a type, return if that type is a continuous type
@@ -262,17 +318,6 @@ def is_pandas_categorical(dtype: Any) -> bool:
     return any(isinstance(dtype, c) for c in CATEGORICAL_PANDAS_DTYPES)
 
 
-def string_dtype_to_object(df: dd.DataFrame) -> dd.DataFrame:
-    """
-    Convert string dtype to object dtype
-    """
-    for col in df.columns:
-        if any(isinstance(df[col].dtype, c) for c in STRING_DTYPES):
-            df[col] = df[col].astype(object)
-
-    return df
-
-
 def drop_null(
     var: Union[dd.Series, pd.DataFrame, dd.DataFrame]
 ) -> Union[pd.Series, dd.Series, pd.DataFrame, dd.DataFrame]:
@@ -315,6 +360,10 @@ def get_dtype_cnts_and_num_cols(
             num_cols.append(col)
         elif is_dtype(col_dtype, DateTime()):
             dtype_cnts["DateTime"] += 1
+        elif is_dtype(col_dtype, GeoGraphy()):
+            dtype_cnts["GeoGraphy"] += 1
+        elif is_dtype(col_dtype, GeoPoint()):
+            dtype_cnts["GeoPoint"] += 1
         else:
             raise NotImplementedError
     return dtype_cnts, num_cols
