@@ -4,32 +4,27 @@ from typing import Optional, Tuple
 import dask.array as da
 import dask.dataframe as dd
 
-from ...dtypes import (
-    Continuous,
-    DTypeDef,
-    Nominal,
-    detect_dtype,
-    is_dtype,
-)
+from ...configs import Config
+from ...dtypes_v2 import Continuous, Nominal, GeoGraphy, SmallCardNum, DateTime, DType
 
 LABELS = ["Orignal data", "After drop missing values"]
 
 
 def uni_histogram(
     srs: dd.Series,
-    bins: int,
-    dtype: Optional[DTypeDef] = None,
+    srs_dtype: DType,
+    cfg: Config,
 ) -> Tuple[da.Array, ...]:
     """Calculate "histogram" for both numerical and categorical."""
 
-    if is_dtype(detect_dtype(srs, dtype), Continuous()):
+    if isinstance(srs_dtype, Continuous):
 
-        counts, edges = da.histogram(srs, bins, range=[srs.min(), srs.max()])
+        counts, edges = da.histogram(srs, cfg.hist.bins, (srs.min(), srs.max()))
         centers = (edges[:-1] + edges[1:]) / 2
 
         return counts, centers, edges
 
-    elif is_dtype(detect_dtype(srs, dtype), Nominal()):
+    elif isinstance(srs_dtype, (Nominal, GeoGraphy, SmallCardNum, DateTime)):
         # Dask array's unique is way slower than the values_counts on Series
         # See https://github.com/dask/dask/issues/2851
         # centers, counts = da.unique(arr, return_counts=True)
@@ -46,16 +41,16 @@ def uni_histogram(
 
 def histogram(
     arr: da.Array,
+    eda_dtype: DType,
     bins: Optional[int] = None,
     return_edges: bool = True,
     range: Optional[Tuple[int, int]] = None,  # pylint: disable=redefined-builtin
-    dtype: Optional[DTypeDef] = None,
 ) -> Tuple[da.Array, ...]:
     """Calculate "histogram" for both numerical and categorical."""
     if len(arr.shape) != 1:
         raise ValueError("Histogram only supports 1-d array.")
-
-    if is_dtype(detect_dtype(arr, dtype), Continuous()):
+    srs = dd.from_dask_array(arr)
+    if isinstance(eda_dtype, Continuous):
         if range is not None:
             minimum, maximum = range
         else:
@@ -70,12 +65,11 @@ def histogram(
         if not return_edges:
             return counts, centers
         return counts, centers, edges
-    elif is_dtype(detect_dtype(arr, dtype), Nominal()):
+    elif isinstance(eda_dtype, (Nominal, GeoGraphy, SmallCardNum, DateTime)):
         # Dask array's unique is way slower than the values_counts on Series
         # See https://github.com/dask/dask/issues/2851
         # centers, counts = da.unique(arr, return_counts=True)
 
-        srs = dd.from_dask_array(arr)
         value_counts = srs.value_counts()
 
         counts = value_counts.to_dask_array()
@@ -83,4 +77,4 @@ def histogram(
 
         return (counts, centers)
     else:
-        raise ValueError(f"Unsupported dtype {arr.dtype}")
+        raise ValueError(f"Unsupported dtype {eda_dtype}")
