@@ -1,31 +1,27 @@
-"""This module implements the plot_missing(df) function's
+"""This module implements the plot_missing(df, x) function's
 calculating intermediate part
 """
-from typing import Any, Generator, List, Optional
+from typing import Any, Generator, List
 
 import numpy as np
 import pandas as pd
 
-from ...data_array import DataArray
-from ...dtypes import (
-    DTypeDef,
-    Nominal,
-    detect_dtype,
-    is_dtype,
-)
+from ...configs import Config
+from ...eda_frame import EDAFrame
+from ...dtypes_v2 import Continuous, Nominal, GeoGraphy, SmallCardNum, DateTime
 from ...intermediate import ColumnsMetadata, Intermediate
 from ...staged import staged
 from .common import LABELS, uni_histogram
 
 
 def _compute_missing_univariate(  # pylint: disable=too-many-locals
-    df: DataArray,
+    df: EDAFrame,
     x: str,
-    bins: int,
-    dtype: Optional[DTypeDef] = None,
+    cfg: Config,
 ) -> Generator[Any, Any, Intermediate]:
     """Calculate the distribution change on other columns when
     the missing values in x is dropped."""
+    # pylint: disable = too-many-boolean-expressions
 
     # dataframe with all rows where column x is null removed
     ddf = df.frame[~df.frame[x].isna()]
@@ -33,13 +29,29 @@ def _compute_missing_univariate(  # pylint: disable=too-many-locals
     hists = {}
 
     for col in df.columns:
-        if col == x:
+        col_dtype = df.get_eda_dtype(col)
+        if (
+            col == x
+            or (
+                isinstance(col_dtype, (Nominal, GeoGraphy, SmallCardNum, DateTime))
+                and not cfg.bar.enable
+            )
+            or (isinstance(col_dtype, Continuous) and not cfg.hist.enable)
+        ):
             continue
 
-        srs0 = df.frame[col].dropna()  # series from original dataframe
-        srs1 = ddf[col].dropna()  # series with null rows from col x removed
+        if isinstance(col_dtype, (SmallCardNum, DateTime)):
+            srs0 = df.frame[col].dropna().astype(str)  # series from original dataframe
+            srs1 = ddf[col].dropna().astype(str)  # series with null rows from col x removed
+        elif isinstance(col_dtype, (GeoGraphy, Nominal, Continuous)):
+            # Geograph, Nominal should be transformed to str when constructing edaframe.
+            # Here we do not need to transform them again.
+            srs0 = df.frame[col].dropna()
+            srs1 = ddf[col].dropna()
+        else:
+            raise ValueError(f"unprocessed type:{col_dtype}")
 
-        hists[col] = [uni_histogram(srs, bins=bins, dtype=dtype) for srs in [srs0, srs1]]
+        hists[col] = [uni_histogram(srs, col_dtype, cfg) for srs in [srs0, srs1]]
 
     ### Lazy Region End
     hists = yield hists
@@ -75,14 +87,18 @@ def _compute_missing_univariate(  # pylint: disable=too-many-locals
 
         # If the cardinality of a categorical column is too large,
         # we show the top `num_bins` values, sorted by their count before drop
-        if len(counts[0]) > bins and is_dtype(detect_dtype(df.frame[col_name], dtype), Nominal()):
+        col_dtype = df.get_eda_dtype(col_name)
+        if len(counts[0]) > cfg.bar.bars and (
+            isinstance(col_dtype, (Nominal, GeoGraphy, SmallCardNum, DateTime))
+        ):
             sortidx = np.argsort(-counts[0])
-            selected_xs = xs[0][sortidx[:bins]]
+            selected_xs = xs[0][sortidx[: cfg.bar.bars]]
             ret_df = ret_df[ret_df["x"].isin(selected_xs)]
-            meta[col_name, "partial"] = (bins, len(counts[0]))
+            meta[col_name, "shown"] = cfg.bar.bars
         else:
-            meta[col_name, "partial"] = (len(counts[0]), len(counts[0]))
-        meta[col_name, "dtype"] = detect_dtype(df.frame[col_name], dtype)
+            meta[col_name, "shown"] = len(counts[0])
+        meta[col_name, "total"] = len(counts[0])
+        meta[col_name, "dtype"] = col_dtype
         dfs[col_name] = ret_df
 
     return Intermediate(data=dfs, x=x, meta=meta, visual_type="missing_impact_1vn")
