@@ -14,19 +14,18 @@ from bokeh.models import (
     ColumnDataSource,
     CustomJSHover,
     FactorRange,
-    FuncTickFormatter,
     HoverTool,
     LinearColorMapper,
     NumeralTickFormatter,
     Panel,
     PrintfTickFormatter,
     Range1d,
-    Title,
 )
 from bokeh.plotting import Figure
 
 from ...errors import UnreachableError
-from ..dtypes import Continuous, Nominal, drop_null, is_dtype
+from ..configs import Config
+from ..dtypes_v2 import Continuous, Nominal, GeoGraphy, SmallCardNum, drop_null, DateTime
 from ..intermediate import ColumnMetadata, Intermediate
 from ..palette import CATEGORY10, CATEGORY20, GREYS256, RDBU
 from ..utils import cut_long_name, fuse_missing_perc, relocate_legend
@@ -35,20 +34,17 @@ from .compute.common import LABELS
 __all__ = ["render_missing"]
 
 
-def render_missing(
-    itmdt: Intermediate,
-    plot_width: int = 400,
-    plot_height: int = 400,
-) -> Dict[str, Any]:
+def render_missing(itmdt: Intermediate, cfg: Config) -> Dict[str, Any]:
     """
-    @Jinglin write here
+    Render the visualizations from plot_missing
     """
+
     if itmdt.visual_type == "missing_impact":
-        return render_missing_impact(itmdt, plot_width, plot_height)
+        return render_missing_impact(itmdt, cfg)
     elif itmdt.visual_type == "missing_impact_1vn":
-        return render_missing_impact_1vn(itmdt, plot_width - 100, plot_height - 100)
+        return render_missing_impact_1vn(itmdt, cfg)
     elif itmdt.visual_type == "missing_impact_1v1":
-        return render_missing_impact_1v1(itmdt, plot_width, plot_height)
+        return render_missing_impact_1v1(itmdt, cfg)
     else:
         raise UnreachableError
 
@@ -63,20 +59,6 @@ def tweak_figure(fig: Figure) -> Figure:
     fig.axis.major_label_text_font_size = "9pt"
     fig.axis.major_label_standoff = 0
     fig.xaxis.major_label_orientation = np.pi / 3
-    # truncate axis tick values
-    format_js = """
-        if (tick.toString().length > 15) {
-            if (typeof tick === 'string') {
-                return tick.toString().substring(0, 13) + '...';
-            } else {
-                return tick.toPrecision(1);
-            }
-        } else {
-            return tick;
-        }
-    """
-    fig.xaxis.formatter = FuncTickFormatter(code=format_js)
-    fig.yaxis.formatter = FuncTickFormatter(code=format_js)
 
     return fig
 
@@ -95,10 +77,10 @@ def render_dist(
     tooltips = [
         (x, "@x"),
         (typ.upper(), f"@{{{typ}}}"),
-        ("Label", "@label"),
+        ("Label", f"@{{{typ}}}_label"),
     ]
     y_range = Range1d(0, df[typ].max() * 1.01)
-    x_range = Range1d(0, df["x"].max() * 1.01)
+    x_range = Range1d(0, df[f"x_{typ}"].max() * 1.01)
 
     fig = tweak_figure(
         Figure(
@@ -106,15 +88,16 @@ def render_dist(
             y_range=y_range,
             plot_width=plot_width,
             plot_height=plot_height,
+            title=" ",
             tools="hover",
             toolbar_location=None,
             tooltips=tooltips,
         )
     )
     for idx, label in enumerate(LABELS):
-        group = df[df["label"] == label]
+        group = df[df[f"{typ}_label"] == label]
         fig.line(
-            x="x",
+            x=f"x_{typ}",
             y=typ,
             source=group,
             color=CATEGORY10[idx],
@@ -126,7 +109,7 @@ def render_dist(
     return fig
 
 
-def render_hist(  # pylint: disable=too-many-arguments
+def render_hist(
     df: pd.DataFrame,
     x: str,
     meta: ColumnMetadata,
@@ -137,13 +120,16 @@ def render_hist(  # pylint: disable=too-many-arguments
     """
     Render a histogram
     """
-    if is_dtype(meta["dtype"], Nominal()):
+    # pylint: disable=too-many-arguments
+    # pylint: disable=too-many-locals
+
+    if isinstance(meta["dtype"], (Nominal, GeoGraphy, SmallCardNum, DateTime)):
         tooltips = [
             (x, "@x"),
             ("Count", "@count"),
             ("Label", "@label"),
         ]
-    else:
+    elif isinstance(meta["dtype"], Continuous):
         df = df.copy()
         df["repr"] = [f"[{row.lower_bound:.0f}~{row.upper_bound:.0f})" for row in df.itertuples()]
 
@@ -152,20 +138,28 @@ def render_hist(  # pylint: disable=too-many-arguments
             ("Frequency", "@count"),
             ("Label", "@label"),
         ]
+    else:
+        mtype = type(meta["dtype"])
+        raise ValueError(f"unprocessed data type:{mtype}, col:{x}")
 
+    cols = [f"{col[:12]}..." if isinstance(col, str) and len(col) > 18 else col for col in df["x"]]
+    df["x"] = cols
     cmapper = CategoricalColorMapper(palette=CATEGORY10, factors=LABELS)
 
-    if is_dtype(meta["dtype"], Nominal()):
+    if isinstance(meta["dtype"], (Nominal, GeoGraphy, SmallCardNum, DateTime)):
         radius = 0.99
 
         # Inputs of FactorRange() have to be sequence of strings,
         # object only contains numbers can cause errors.(Issue#98).
         df["x"] = df["x"].astype("str")
         x_range = FactorRange(*df["x"].unique())
-    else:
+    elif isinstance(meta["dtype"], Continuous):
 
         radius = df["x"][1] - df["x"][0]
         x_range = Range1d(df["x"].min() - radius, df["x"].max() + radius)
+    else:
+        mtype = type(meta["dtype"])
+        raise ValueError(f"unprocessed data type:{mtype}, col:{x}")
 
     y_range = Range1d(0, df["count"].max() * 1.05)
 
@@ -175,6 +169,7 @@ def render_hist(  # pylint: disable=too-many-arguments
             y_range=y_range,
             plot_width=plot_width,
             plot_height=plot_height,
+            title=" " if show_legend else "",
             tools="hover",
             toolbar_location=None,
             tooltips=tooltips,
@@ -193,7 +188,7 @@ def render_hist(  # pylint: disable=too-many-arguments
 
         relocate_legend(fig, "left")
     else:
-        shown, total = meta["partial"]
+        shown, total = meta["shown"], meta["total"]
         if shown != total:
             fig.xaxis.axis_label = f"Top {shown} out of {total}"
             fig.xaxis.axis_label_standoff = 0
@@ -227,6 +222,7 @@ def render_boxwhisker(df: pd.DataFrame, plot_width: int, plot_height: int) -> Fi
             x_range=df["label"].unique(),
             plot_width=plot_width,
             plot_height=plot_height,
+            title=" ",
             tools="",
             toolbar_location=None,
             tooltips=tooltips,
@@ -310,37 +306,53 @@ def create_color_mapper_heatmap(
     return mapper, colorbar
 
 
-def render_missing_impact(itmdt: Intermediate, plot_width: int, plot_height: int) -> Dict[str, Any]:
+def render_missing_impact(itmdt: Intermediate, cfg: Config) -> Dict[str, Any]:
     """
     Render correlation heatmaps in to tabs
     """
+    plot_width = cfg.plot.width if cfg.plot.width is not None else 500
+    plot_height = cfg.plot.height if cfg.plot.height is not None else 500
+
     tabs: List[Panel] = []
-    fig_barchart = render_bar_chart(itmdt["data_bars"], "linear", plot_width, plot_height)
-    tabs.append(Panel(child=row(fig_barchart), title="Bar Chart"))
+    htgs: Dict[str, List[Tuple[str, str]]] = {}
 
-    fig_spectrum = render_missing_spectrum(
-        itmdt["data_spectrum"], itmdt["data_total_missing"], plot_width, plot_height
-    )
-    tabs.append(Panel(child=row(fig_spectrum), title="Spectrum"))
+    if cfg.bar.enable:
+        fig_barchart = render_bar_chart(itmdt["data_bars"], "linear", plot_width, plot_height)
+        tabs.append(Panel(child=row(fig_barchart), title="Bar Chart"))
+        htgs["Bar Chart"] = cfg.bar.missing_how_to_guide(plot_height, plot_width)
 
-    fig_heatmap = render_heatmaps(itmdt["data_heatmap"], plot_width, plot_height)
-    tabs.append(Panel(child=row(fig_heatmap), title="Heatmap"))
+    if cfg.spectrum.enable:
+        fig_spectrum = render_missing_spectrum(
+            itmdt["data_spectrum"], itmdt["data_total_missing"], plot_width, plot_height
+        )
+        tabs.append(Panel(child=row(fig_spectrum), title="Spectrum"))
+        htgs["Spectrum"] = cfg.spectrum.how_to_guide(plot_height, plot_width)
 
-    fig_dendrogram = render_dendrogram(itmdt["data_dendrogram"], plot_width, plot_height)
-    tabs.append(Panel(child=row(fig_dendrogram), title="Dendrogram"))
+    if cfg.heatmap.enable:
+        fig_heatmap = render_heatmaps(itmdt["data_heatmap"], plot_width, plot_height)
+        tabs.append(Panel(child=row(fig_heatmap), title="Heat Map"))
+        htgs["Heat Map"] = cfg.heatmap.missing_how_to_guide(plot_height, plot_width)
 
-    stat_dict = {name: itmdt["missing_stat"][name] for name in itmdt["missing_stat"]}
+    # itmdt["data_dendrogram"] is None when dataframe has only one column so that won't be displayed
+    if cfg.dendro.enable and itmdt["ncols"] > 1:
+        fig_dendrogram = render_dendrogram(itmdt["data_dendrogram"], plot_width, plot_height)
+        tabs.append(Panel(child=row(fig_dendrogram), title="Dendrogram"))
+        htgs["Dendrogram"] = cfg.dendro.how_to_guide(plot_height, plot_width)
+
+    if cfg.stats.enable:
+        stat_dict = {name: itmdt["missing_stat"][name] for name in itmdt["missing_stat"]}
 
     return {
         "insights": itmdt["insights"],
-        "tabledata": {"Missing Statistics": stat_dict},
+        "tabledata": {"Missing Statistics": stat_dict} if cfg.stats.enable else {},
         "layout": [panel.child.children[0] for panel in tabs],
         "meta": [panel.title for panel in tabs],
-        "container_width": plot_width + 160,
+        "container_width": max([panel.child.children[0].plot_width for panel in tabs]),
+        "how_to_guide": htgs,
     }
 
 
-def render_heatmaps(df: Optional[pd.DataFrame], plot_width: int, plot_height: int) -> Figure:
+def render_heatmaps(data: Optional[pd.DataFrame], plot_width: int, plot_height: int) -> Figure:
     """
     Render missing heatmaps in to tabs
     """
@@ -356,6 +368,7 @@ def render_heatmaps(df: Optional[pd.DataFrame], plot_width: int, plot_height: in
             plot_width=plot_width,
             plot_height=plot_height,
             x_axis_location="below",
+            title=" ",
             tools="hover",
             toolbar_location=None,
             background_fill_color="#fafafa",
@@ -366,8 +379,10 @@ def render_heatmaps(df: Optional[pd.DataFrame], plot_width: int, plot_height: in
         fig.rect(x=0, y=0, width=0, height=0)
         return fig
 
-    if df is not None:
-
+    if data is not None:
+        df = data.copy()
+        df.columns = list(map(cut_long_name, df.columns))
+        df.index = list(map(cut_long_name, df.index))
         df = df.where(np.triu(np.ones(df.shape)).astype(np.bool)).T  # pylint: disable=no-member
 
         if df.size != 0:
@@ -379,10 +394,6 @@ def render_heatmaps(df: Optional[pd.DataFrame], plot_width: int, plot_height: in
             df = df[df["x"] != df["y"]]
             df = drop_null(df)
 
-            # in case of numerical column names
-            df["x"] = df["x"].apply(str)
-            df["y"] = df["y"].apply(str)
-
             fig = Figure(
                 x_range=x_range,
                 y_range=y_range,
@@ -414,7 +425,6 @@ def render_heatmaps(df: Optional[pd.DataFrame], plot_width: int, plot_height: in
     fig.grid.grid_line_color = None
     fig.axis.axis_line_color = None
     fig.add_layout(color_bar, "left")
-    fig.frame_width = plot_width
     return fig
 
 
@@ -428,6 +438,7 @@ def render_bar_chart(
     Render a bar chart for the missing and present values
     """
     pres_cnts, null_cnts, cols = data
+    cols = [f"{col[:13]}...{col[len(col)-3:]}" if len(col) > 18 else col for col in cols]
     df = pd.DataFrame({"Present": pres_cnts, "Missing": null_cnts}, index=cols)
 
     if len(df) > 20:
@@ -452,7 +463,6 @@ def render_bar_chart(
         source=df,
         legend_label=list(df.columns),
     )
-
     # hover tool with count and percent
     formatter = CustomJSHover(
         args=dict(source=ColumnDataSource(df)),
@@ -484,7 +494,6 @@ def render_bar_chart(
     fig.yaxis.axis_label = "Row Count"
     tweak_figure(fig)
     relocate_legend(fig, "left")
-    fig.frame_width = plot_width
 
     return fig
 
@@ -551,7 +560,6 @@ def render_missing_spectrum(
         line_color=None,
     )
     fig.add_layout(color_bar, "left")
-    fig.frame_width = plot_width
     return fig
 
 
@@ -601,28 +609,37 @@ def render_dendrogram(dend: Dict["str", Any], plot_width: int, plot_height: int)
     fig.xaxis.major_label_orientation = np.pi / 3
     fig.yaxis.axis_label = "Average Distance Between Clusters"
     fig.grid.visible = False
-    fig.frame_width = plot_width
     return fig
 
 
-def render_missing_impact_1vn(
-    itmdt: Intermediate,
-    plot_width: int,
-    plot_height: int,
-) -> Dict[str, Any]:
+def render_missing_impact_1vn(itmdt: Intermediate, cfg: Config) -> Dict[str, Any]:
     """
     Render the plot from `plot_missing(df, "x")`
     """
+    # pylint: disable = too-many-locals
+    plot_width = cfg.plot.width if cfg.plot.width is not None else 300
+    plot_height = cfg.plot.height if cfg.plot.height is not None else 300
 
     dfs = itmdt["data"]
     x = itmdt["x"]
     meta = itmdt["meta"]
     panels = []
+    htgs: Dict[str, List[Tuple[str, str]]] = {}
+    titles: List[str] = []
     for col, df in dfs.items():
+        title = f"Missing impact of {x} by {col}"
         fig = render_hist(df, col, meta[col], plot_width, plot_height, False)
         fig.frame_height = plot_height
-        fig.title = Title(text=f"Missing impact of {x} by {col}")
         panels.append(Panel(child=fig, title=col))
+
+        if isinstance(meta[col]["dtype"], (Nominal, GeoGraphy, SmallCardNum, DateTime)):
+            htgs[title] = cfg.bar.grid_how_to_guide()
+        elif isinstance(meta[col]["dtype"], Continuous):
+            htgs[title] = cfg.hist.grid_how_to_guide()
+        else:
+            mtype = type(meta[col]["dtype"])
+            raise ValueError(f"unprocessed type:{mtype}")
+        titles.append(title)
     legend_colors = [CATEGORY10[count] for count in range(len(LABELS))]
     return {
         "layout": [panel.child for panel in panels],
@@ -630,34 +647,40 @@ def render_missing_impact_1vn(
         "legend_labels": [
             {"label": label, "color": color} for label, color in zip(LABELS, legend_colors)
         ],
+        "meta": titles,
+        "how_to_guide": htgs,
     }
 
 
-def render_missing_impact_1v1(
-    itmdt: Intermediate,
-    plot_width: int,
-    plot_height: int,
-) -> Dict[str, Any]:
+def render_missing_impact_1v1(itmdt: Intermediate, cfg: Config) -> Dict[str, Any]:
     """
     Render the plot from `plot_missing(df, "x", "y")`
     """
-    x, y = itmdt["x"], itmdt["y"]
-    meta = itmdt["meta"]
+    plot_width = cfg.plot.width if cfg.plot.width is not None else 400
+    plot_height = cfg.plot.height if cfg.plot.height is not None else 400
 
-    if is_dtype(meta["dtype"], Continuous()):
+    x, y, meta = itmdt["x"], itmdt["y"], itmdt["meta"]
+    htgs: Dict[str, List[Tuple[str, str]]] = {}
+
+    if isinstance(meta["dtype"], Continuous):
         panels = []
 
-        fig = render_hist(itmdt["hist"], y, meta, plot_width, plot_height, True)
-        panels.append(Panel(child=fig, title="Histogram"))
-
-        fig = render_dist(itmdt["dist"], y, "pdf", plot_width, plot_height)
-        panels.append(Panel(child=fig, title="PDF"))
-
-        fig = render_dist(itmdt["dist"], y, "cdf", plot_width, plot_height)
-        panels.append(Panel(child=fig, title="CDF"))
-
-        fig = render_boxwhisker(itmdt["box"], plot_width, plot_height)
-        panels.append(Panel(child=fig, title="Box"))
+        if cfg.hist.enable:
+            fig = render_hist(itmdt["hist"], y, meta, plot_width, plot_height, True)
+            panels.append(Panel(child=fig, title="Histogram"))
+            htgs["Histogram"] = cfg.hist.how_to_guide(plot_height, plot_width)
+        if cfg.pdf.enable:
+            fig = render_dist(itmdt["dist"], y, "pdf", plot_width, plot_height)
+            panels.append(Panel(child=fig, title="PDF"))
+            htgs["PDF"] = cfg.pdf.how_to_guide(plot_height, plot_width)
+        if cfg.cdf.enable:
+            fig = render_dist(itmdt["dist"], y, "cdf", plot_width, plot_height)
+            panels.append(Panel(child=fig, title="CDF"))
+            htgs["CDF"] = cfg.cdf.how_to_guide(plot_height, plot_width)
+        if cfg.box.enable:
+            fig = render_boxwhisker(itmdt["box"], plot_width, plot_height)
+            panels.append(Panel(child=fig, title="Box Plot"))
+            htgs["Box Plot"] = cfg.box.univar_how_to_guide(plot_height, plot_width)
 
         for panel in panels:
             panel.child.frame_width = plot_width
@@ -665,14 +688,23 @@ def render_missing_impact_1v1(
         return {
             "layout": [panel.child for panel in panels],
             "meta": [panel.title for panel in panels],
-            "container_width": plot_width + 240,
+            "container_width": max([panel.child.plot_width for panel in panels]),
+            "how_to_guide": htgs,
         }
-    else:
+    elif isinstance(meta["dtype"], (Nominal, SmallCardNum, GeoGraphy, DateTime)):
         fig = render_hist(itmdt["hist"], y, meta, plot_width, plot_height, True)
-        fig.frame_width = plot_width
-        shown, total = meta["partial"]
+        shown, total = meta["shown"], meta["total"]
         if shown != total:
             _title = f"Missing impact of {x} by ({shown} out of {total}) {y}"
         else:
             _title = f"Missing impact of {x} by {y}"
-        return {"layout": [fig], "meta": [_title], "container_width": plot_width + 240}
+        htgs[_title] = cfg.bar.how_to_guide(plot_height, plot_width)
+        return {
+            "layout": [fig],
+            "meta": [_title],
+            "container_width": fig.plot_width,
+            "how_to_guide": htgs,
+        }
+    else:
+        mtype = type(meta["dtype"])
+        raise ValueError(f"unsupported type:{mtype}")
